# Copyright (c) 2015 Nicolas JOUANIN
#
# See the file license.txt for copying permission.
import anyio
import logging
import unittest
from unittest.mock import patch, call, MagicMock

from distmqtt.adapters import StreamAdapter
from distmqtt.broker import (
    EVENT_BROKER_PRE_START,
    EVENT_BROKER_POST_START,
    EVENT_BROKER_PRE_SHUTDOWN,
    EVENT_BROKER_POST_SHUTDOWN,
    EVENT_BROKER_CLIENT_CONNECTED,
    EVENT_BROKER_CLIENT_DISCONNECTED,
    EVENT_BROKER_CLIENT_SUBSCRIBED,
    EVENT_BROKER_CLIENT_UNSUBSCRIBED,
    EVENT_BROKER_MESSAGE_RECEIVED,
    create_broker)
from distmqtt.client import open_mqttclient, ConnectException
from distmqtt.mqtt import (
    ConnectPacket, ConnackPacket, PublishPacket, PubrecPacket,
    PubrelPacket, PubcompPacket, DisconnectPacket)
from distmqtt.mqtt.connect import ConnectVariableHeader, ConnectPayload
from distmqtt.mqtt.constants import QOS_0, QOS_1, QOS_2


formatter = "%(asctime)s %(name)s:%(lineno)d %(levelname)s - %(message)s"
logging.basicConfig(level=logging.INFO, format=formatter)
log = logging.getLogger(__name__)

test_config = {
    'listeners': {
        'default': {
            'type': 'tcp',
            'bind': '127.0.0.1:1883',
            'max_connections': 10
        },
    },
    'sys_interval': 0,
    'auth': {
        'allow-anonymous': True,
    }
}


class AsyncMock(MagicMock):
    def __await__(self):
        async def foo():
            return self
        return foo().__await__()

class BrokerTest(unittest.TestCase):
    @patch('distmqtt.broker.PluginManager', new_callable=AsyncMock)
    def test_start_stop(self, MockPluginManager):
        async def test_coro():
            async with create_broker(test_config, plugin_namespace="distmqtt.test.plugins") as broker:
                broker.plugins_manager._tg = broker._tg
                self.assertTrue(broker.transitions.is_started())
                self.assertDictEqual(broker._sessions, {})
                self.assertIn('default', broker._servers)
                MockPluginManager.assert_has_calls(
                    [call().fire_event(EVENT_BROKER_PRE_START),
                        call().fire_event(EVENT_BROKER_POST_START)], any_order=True)
                MockPluginManager.reset_mock()
            MockPluginManager.assert_has_calls(
                [call().fire_event(EVENT_BROKER_PRE_SHUTDOWN),
                    call().fire_event(EVENT_BROKER_POST_SHUTDOWN)], any_order=True)
            self.assertTrue(broker.transitions.is_stopped())

        anyio.run(test_coro, backend='trio')

    @patch('distmqtt.broker.PluginManager', new_callable=AsyncMock)
    def test_client_connect(self, MockPluginManager):
        async def test_coro():
            async with create_broker(test_config, plugin_namespace="distmqtt.test.plugins") as broker:
                broker.plugins_manager._tg = broker._tg
                self.assertTrue(broker.transitions.is_started())
                async with open_mqttclient() as client:
                    ret = await client.connect('mqtt://127.0.0.1/')
                    self.assertEqual(ret, 0)
                    self.assertEqual(len(broker._sessions), 1)
                    self.assertIn(client.session.client_id, broker._sessions)
                await anyio.sleep(0.1) # let the broker task process the packet
            self.assertTrue(broker.transitions.is_stopped())
            self.assertDictEqual(broker._sessions, {})
            MockPluginManager.assert_has_calls(
                [call().fire_event(EVENT_BROKER_CLIENT_CONNECTED, client_id=client.session.client_id),
                    call().fire_event(EVENT_BROKER_CLIENT_DISCONNECTED, client_id=client.session.client_id)],
                any_order=True)

        anyio.run(test_coro)

    @patch('distmqtt.broker.PluginManager', new_callable=AsyncMock)
    def test_client_connect_will_flag(self, MockPluginManager):
        async def test_coro():
            async with create_broker(test_config, plugin_namespace="distmqtt.test.plugins") as broker:
                broker.plugins_manager._tg = broker._tg
                self.assertTrue(broker.transitions.is_started())

                async with await anyio.connect_tcp("127.0.0.1", 1883) as conn:
                    stream = StreamAdapter(conn)

                    vh = ConnectVariableHeader()
                    payload = ConnectPayload()

                    vh.keep_alive = 10
                    vh.clean_session_flag = False
                    vh.will_retain_flag = False
                    vh.will_flag = True
                    vh.will_qos = QOS_0
                    payload.client_id = 'test_id'
                    payload.will_message = b'test'
                    payload.will_topic = '/topic'
                    connect = ConnectPacket(vh=vh, payload=payload)
                    await connect.to_stream(stream)
                    await ConnackPacket.from_stream(stream)


                    disconnect = DisconnectPacket()
                    await disconnect.to_stream(stream)

            self.assertTrue(broker.transitions.is_stopped())
            self.assertDictEqual(broker._sessions, {})

        anyio.run(test_coro, backend="trio")

    @patch('distmqtt.broker.PluginManager', new_callable=AsyncMock)
    def test_client_connect_clean_session_false(self, MockPluginManager):
        async def test_coro():
            async with create_broker(test_config, plugin_namespace="distmqtt.test.plugins") as broker:
                broker.plugins_manager._tg = broker._tg
                self.assertTrue(broker.transitions.is_started())
                async with open_mqttclient(client_id="", config={'auto_reconnect': False}) as client:
                    return_code = None
                    try:
                        await client.connect('mqtt://127.0.0.1/', cleansession=False)
                    except ConnectException as ce:
                        return_code = ce.return_code
                    self.assertEqual(return_code, 0x02)
                    self.assertNotIn(client.session.client_id, broker._sessions)

        anyio.run(test_coro)

    @patch('distmqtt.broker.PluginManager', new_callable=AsyncMock)
    def test_client_subscribe(self, MockPluginManager):
        async def test_coro():
            async with create_broker(test_config, plugin_namespace="distmqtt.test.plugins") as broker:
                broker.plugins_manager._tg = broker._tg
                self.assertTrue(broker.transitions.is_started())
                async with open_mqttclient() as client:
                    ret = await client.connect('mqtt://127.0.0.1/')
                    self.assertEqual(ret, 0)
                    await client.subscribe([('/topic', QOS_0)])

                    # Test if the client test client subscription is registered
                    self.assertIn('/topic', broker._subscriptions)
                    subs = broker._subscriptions['/topic']
                    self.assertEqual(len(subs), 1)
                    (s, qos) = subs[0]
                    self.assertEqual(s, client.session)
                    self.assertEqual(qos, QOS_0)

            self.assertTrue(broker.transitions.is_stopped())
            MockPluginManager.assert_has_calls(
                [call().fire_event(EVENT_BROKER_CLIENT_SUBSCRIBED,
                                    client_id=client.session.client_id,
                                    topic='/topic', qos=QOS_0)], any_order=True)

        anyio.run(test_coro, backend="trio")

    @patch('distmqtt.broker.PluginManager', new_callable=AsyncMock)
    def test_client_subscribe_twice(self, MockPluginManager):
        async def test_coro():
            async with create_broker(test_config, plugin_namespace="distmqtt.test.plugins") as broker:
                broker.plugins_manager._tg = broker._tg
                self.assertTrue(broker.transitions.is_started())
                async with open_mqttclient() as client:
                    ret = await client.connect('mqtt://127.0.0.1/')
                    self.assertEqual(ret, 0)
                    await client.subscribe([('/topic', QOS_0)])

                    # Test if the client test client subscription is registered
                    self.assertIn('/topic', broker._subscriptions)
                    subs = broker._subscriptions['/topic']
                    self.assertEqual(len(subs), 1)
                    (s, qos) = subs[0]
                    self.assertEqual(s, client.session)
                    self.assertEqual(qos, QOS_0)

                    await client.subscribe([('/topic', QOS_0)])
                    self.assertEqual(len(subs), 1)
                    (s, qos) = subs[0]
                    self.assertEqual(s, client.session)
                    self.assertEqual(qos, QOS_0)

            self.assertTrue(broker.transitions.is_stopped())
            MockPluginManager.assert_has_calls(
                [call().fire_event(EVENT_BROKER_CLIENT_SUBSCRIBED,
                                    client_id=client.session.client_id,
                                    topic='/topic', qos=QOS_0)], any_order=True)

        anyio.run(test_coro, backend="trio")

    @patch('distmqtt.broker.PluginManager', new_callable=AsyncMock)
    def test_client_unsubscribe(self, MockPluginManager):
        async def test_coro():
            async with create_broker(test_config, plugin_namespace="distmqtt.test.plugins") as broker:
                broker.plugins_manager._tg = broker._tg
                self.assertTrue(broker.transitions.is_started())
                async with open_mqttclient() as client:
                    ret = await client.connect('mqtt://127.0.0.1/')
                    self.assertEqual(ret, 0)
                    await client.subscribe([('/topic', QOS_0)])

                    # Test if the client test client subscription is registered
                    self.assertIn('/topic', broker._subscriptions)
                    subs = broker._subscriptions['/topic']
                    self.assertEqual(len(subs), 1)
                    (s, qos) = subs[0]
                    self.assertEqual(s, client.session)
                    self.assertEqual(qos, QOS_0)

                    await client.unsubscribe(['/topic'])
                    self.assertEqual(broker._subscriptions['/topic'], [])
            self.assertTrue(broker.transitions.is_stopped())
            MockPluginManager.assert_has_calls(
                [
                    call().fire_event(EVENT_BROKER_CLIENT_SUBSCRIBED,
                                        client_id=client.session.client_id,
                                        topic='/topic', qos=QOS_0),
                    call().fire_event(EVENT_BROKER_CLIENT_UNSUBSCRIBED,
                                        client_id=client.session.client_id,
                                        topic='/topic')
                ], any_order=True)

        anyio.run(test_coro, backend="trio")

    @patch('distmqtt.broker.PluginManager', new_callable=AsyncMock)
    def test_client_publish(self, MockPluginManager):
        async def test_coro():
            async with create_broker(test_config, plugin_namespace="distmqtt.test.plugins") as broker:
                broker.plugins_manager._tg = broker._tg
                self.assertTrue(broker.transitions.is_started())
                async with open_mqttclient() as pub_client:
                    ret = await pub_client.connect('mqtt://127.0.0.1/')
                    self.assertEqual(ret, 0)

                    ret_message = await pub_client.publish('/topic', b'data', QOS_0)
                await anyio.sleep(0.1) # let the broker task process the packet
                self.assertEqual(broker._retained_messages, {})

            self.assertTrue(broker.transitions.is_stopped())
            MockPluginManager.assert_has_calls(
                [
                    call().fire_event(EVENT_BROKER_MESSAGE_RECEIVED,
                                        client_id=pub_client.session.client_id,
                                        message=ret_message),
                ], any_order=True)

        anyio.run(test_coro)

    #@patch('distmqtt.broker.PluginManager', new_callable=AsyncMock)
    def test_client_publish_dup(self):
        async def test_coro():
            async with create_broker(test_config, plugin_namespace="distmqtt.test.plugins") as broker:
                broker.plugins_manager._tg = broker._tg
                self.assertTrue(broker.transitions.is_started())

                async with await anyio.connect_tcp("127.0.0.1", 1883) as conn:
                    stream = StreamAdapter(conn)

                    vh = ConnectVariableHeader()
                    payload = ConnectPayload()

                    vh.keep_alive = 10
                    vh.clean_session_flag = False
                    vh.will_retain_flag = False
                    payload.client_id = 'test_id'
                    connect = ConnectPacket(vh=vh, payload=payload)
                    await connect.to_stream(stream)
                    await ConnackPacket.from_stream(stream)

                    publish_1 = PublishPacket.build('/test', b'data', 1, False, QOS_2, False)
                    await publish_1.to_stream(stream)
                    await PubrecPacket.from_stream(stream)

                    publish_dup = PublishPacket.build('/test', b'data', 1, True, QOS_2, False)
                    await publish_dup.to_stream(stream)
                    await PubrecPacket.from_stream(stream)
                    pubrel = PubrelPacket.build(1)
                    await pubrel.to_stream(stream)
                    #await PubcompPacket.from_stream(stream)

                    disconnect = DisconnectPacket()
                    await disconnect.to_stream(stream)

        anyio.run(test_coro, backend="trio")

    @patch('distmqtt.broker.PluginManager', new_callable=AsyncMock)
    def test_client_publish_invalid_topic(self, MockPluginManager):
        async def test_coro():
            async with create_broker(test_config, plugin_namespace="distmqtt.test.plugins") as broker:
                broker.plugins_manager._tg = broker._tg
                self.assertTrue(broker.transitions.is_started())
                async with open_mqttclient() as pub_client:
                    ret = await pub_client.connect('mqtt://127.0.0.1/')
                    self.assertEqual(ret, 0)

                    await pub_client.publish('/+', b'data', QOS_0)

            self.assertTrue(broker.transitions.is_stopped())

        anyio.run(test_coro)

    @patch('distmqtt.broker.PluginManager', new_callable=AsyncMock)
    def test_client_publish_big(self, MockPluginManager):
        async def test_coro():
            async with create_broker(test_config, plugin_namespace="distmqtt.test.plugins") as broker:
                broker.plugins_manager._tg = broker._tg
                self.assertTrue(broker.transitions.is_started())
                async with open_mqttclient() as pub_client:
                    ret = await pub_client.connect('mqtt://127.0.0.1/')
                    self.assertEqual(ret, 0)

                    ret_message = await pub_client.publish('/topic', bytearray(b'\x99' * 256 * 1024), QOS_2)
                self.assertEqual(broker._retained_messages, {})

            self.assertTrue(broker.transitions.is_stopped())
            MockPluginManager.assert_has_calls(
                [
                    call().fire_event(EVENT_BROKER_MESSAGE_RECEIVED,
                                        client_id=pub_client.session.client_id,
                                        message=ret_message),
                ], any_order=True)

        anyio.run(test_coro, backend="trio")

    @patch('distmqtt.broker.PluginManager', new_callable=AsyncMock)
    def test_client_publish_retain(self, MockPluginManager):
        async def test_coro():
            async with create_broker(test_config, plugin_namespace="distmqtt.test.plugins") as broker:
                broker.plugins_manager._tg = broker._tg
                self.assertTrue(broker.transitions.is_started())

                async with open_mqttclient() as pub_client:
                    ret = await pub_client.connect('mqtt://127.0.0.1/')
                    self.assertEqual(ret, 0)
                    await pub_client.publish('/topic', b'data', QOS_0, retain=True)
                await anyio.sleep(0.1) # let the broker task process the packet
                self.assertIn('/topic', broker._retained_messages)
                retained_message = broker._retained_messages['/topic']
                self.assertEqual(retained_message.source_session, pub_client.session)
                self.assertEqual(retained_message.topic, '/topic')
                self.assertEqual(retained_message.data, b'data')
                self.assertEqual(retained_message.qos, QOS_0)
            self.assertTrue(broker.transitions.is_stopped())

        anyio.run(test_coro)

    @patch('distmqtt.broker.PluginManager', new_callable=AsyncMock)
    def test_client_publish_retain_delete(self, MockPluginManager):
        async def test_coro():
            async with create_broker(test_config, plugin_namespace="distmqtt.test.plugins") as broker:
                broker.plugins_manager._tg = broker._tg
                self.assertTrue(broker.transitions.is_started())

                async with open_mqttclient() as pub_client:
                    ret = await pub_client.connect('mqtt://127.0.0.1/')
                    self.assertEqual(ret, 0)
                    await pub_client.publish('/topic', b'', QOS_0, retain=True)
                await anyio.sleep(0.1) # let the broker task process the packet
                self.assertNotIn('/topic', broker._retained_messages)
            self.assertTrue(broker.transitions.is_stopped())

        anyio.run(test_coro)

    @patch('distmqtt.broker.PluginManager', new_callable=AsyncMock)
    def test_client_subscribe_publish(self, MockPluginManager):
        async def test_coro():
            async with create_broker(test_config, plugin_namespace="distmqtt.test.plugins") as broker:
                broker.plugins_manager._tg = broker._tg
                self.assertTrue(broker.transitions.is_started())
                async with open_mqttclient() as sub_client:
                    await sub_client.connect('mqtt://127.0.0.1')
                    ret = await sub_client.subscribe([('/qos0', QOS_0), ('/qos1', QOS_1), ('/qos2', QOS_2)])
                    self.assertEqual(ret, [QOS_0, QOS_1, QOS_2])

                    await self._client_publish('/qos0', b'data', QOS_0)
                    await self._client_publish('/qos1', b'data', QOS_1)
                    await self._client_publish('/qos2', b'data', QOS_2)
                    for qos in [QOS_0, QOS_1, QOS_2]:
                        message = await sub_client.deliver_message()
                        self.assertIsNotNone(message)
                        self.assertEqual(message.topic, '/qos%s' % qos)
                        self.assertEqual(message.data, b'data')
                        self.assertEqual(message.qos, qos)
            self.assertTrue(broker.transitions.is_stopped())

        anyio.run(test_coro)

    @patch('distmqtt.broker.PluginManager', new_callable=AsyncMock)
    def test_client_subscribe_invalid(self, MockPluginManager):
        async def test_coro():
            async with create_broker(test_config, plugin_namespace="distmqtt.test.plugins") as broker:
                broker.plugins_manager._tg = broker._tg
                self.assertTrue(broker.transitions.is_started())
                async with open_mqttclient() as sub_client:
                    await sub_client.connect('mqtt://127.0.0.1')
                    ret = await sub_client.subscribe(
                        [('+', QOS_0), ('+/tennis/#', QOS_0), ('sport+', QOS_0), ('sport/+/player1', QOS_0)])
                    self.assertEqual(ret, [QOS_0, QOS_0, 0x80, QOS_0])

            self.assertTrue(broker.transitions.is_stopped())

        anyio.run(test_coro, backend="trio")

    @patch('distmqtt.broker.PluginManager', new_callable=AsyncMock)
    def test_client_subscribe_publish_dollar_topic_1(self, MockPluginManager):
        async def test_coro():
            async with create_broker(test_config, plugin_namespace="distmqtt.test.plugins") as broker:
                broker.plugins_manager._tg = broker._tg
                self.assertTrue(broker.transitions.is_started())
                async with open_mqttclient() as sub_client:
                    await sub_client.connect('mqtt://127.0.0.1')
                    ret = await sub_client.subscribe([('#', QOS_0)])
                    self.assertEqual(ret, [QOS_0])

                    await self._client_publish('/topic', b'data', QOS_0)
                    message = await sub_client.deliver_message()
                    self.assertIsNotNone(message)

                    await self._client_publish('$topic', b'data', QOS_0)
                    message = None
                    with self.assertRaises(TimeoutError):
                        async with anyio.fail_after(2):
                            message = await sub_client.deliver_message()
                    self.assertIsNone(message)
            self.assertTrue(broker.transitions.is_stopped())

        anyio.run(test_coro)

    @patch('distmqtt.broker.PluginManager', new_callable=AsyncMock)
    def test_client_subscribe_publish_dollar_topic_2(self, MockPluginManager):
        async def test_coro():
            async with create_broker(test_config, plugin_namespace="distmqtt.test.plugins") as broker:
                broker.plugins_manager._tg = broker._tg
                self.assertTrue(broker.transitions.is_started())
                async with open_mqttclient() as sub_client:
                    await sub_client.connect('mqtt://127.0.0.1')
                    ret = await sub_client.subscribe([('+/monitor/Clients', QOS_0)])
                    self.assertEqual(ret, [QOS_0])

                    await self._client_publish('/test/monitor/Clients', b'data', QOS_0)
                    message = await sub_client.deliver_message()
                    self.assertIsNotNone(message)

                    await self._client_publish('$SYS/monitor/Clients', b'data', QOS_0)
                    message = None
                    with self.assertRaises(TimeoutError):
                        async with anyio.fail_after(2):
                            message = await sub_client.deliver_message()
                    self.assertIsNone(message)
            self.assertTrue(broker.transitions.is_stopped())

        anyio.run(test_coro)

    @patch('distmqtt.broker.PluginManager', new_callable=AsyncMock)
    def test_client_publish_retain_subscribe(self, MockPluginManager):
        async def test_coro():
            async with create_broker(test_config, plugin_namespace="distmqtt.test.plugins") as broker:
                broker.plugins_manager._tg = broker._tg
                self.assertTrue(broker.transitions.is_started())
                async with open_mqttclient() as sub_client:
                    await sub_client.connect('mqtt://127.0.0.1', cleansession=False)
                    ret = await sub_client.subscribe([('/qos0', QOS_0), ('/qos1', QOS_1), ('/qos2', QOS_2)])
                    self.assertEqual(ret, [QOS_0, QOS_1, QOS_2])
                    await sub_client.disconnect()

                    await self._client_publish('/qos0', b'data', QOS_0, retain=True)
                    await self._client_publish('/qos1', b'data', QOS_1, retain=True)
                    await self._client_publish('/qos2', b'data', QOS_2, retain=True)
                    await sub_client.reconnect()
                    for qos in [QOS_0, QOS_1, QOS_2]:
                        log.debug("TEST QOS: %d" % qos)
                        message = await sub_client.deliver_message()
                        log.debug("Message: " + repr(message.publish_packet))
                        self.assertIsNotNone(message)
                        self.assertEqual(message.topic, '/qos%s' % qos)
                        self.assertEqual(message.data, b'data')
                        self.assertEqual(message.qos, qos)
            self.assertTrue(broker.transitions.is_stopped())

        anyio.run(test_coro)

    async def _client_publish(self, topic, data, qos, retain=False):
        async with open_mqttclient() as pub_client:
            ret = await pub_client.connect('mqtt://127.0.0.1/')
            self.assertEqual(ret, 0)
            ret = await pub_client.publish(topic, data, qos, retain)
        return ret



# Copyright (c) 2015 Nicolas JOUANIN
#
# See the file license.txt for copying permission.
import io
from asyncwebsockets import Websocket
from wsproto.events import CloseConnection, BytesMessage
import anyio
import logging


class BaseAdapter:
    """
    Base class for all network protocol reader adapter.

    Reader adapters are used to adapt read operations on the network depending on the protocol used
    """

    async def read(self, n=-1) -> bytes:
        """
        Read up to n bytes. If n is not provided, or set to -1, read until EOF and return all read bytes.
        If the EOF was received and the internal buffer is empty, return an empty bytes object.
        :return: packet read as bytes data
        """

    def feed_eof(self):
        """
        Acknowleddge EOF
        """

    def write(self, data):
        """
        write some data to the protocol layer
        """

    def get_peer_info(self):
        """
        Return peer socket info (remote address and remote port as tuple
        """

    async def close(self):
        """
        Close the protocol connection
        """


class WebSocketsAdapter(BaseAdapter):
    """
    WebSockets API reader adapter
    """
    def __init__(self, websocket: Websocket):
        self._websocket = websocket
        self._buffer = io.BytesIO(b'')

    async def read(self, n=-1) -> bytes:
        await self._feed_buffer(n)
        data = self._buffer.read(n)
        return data

    async def _feed_buffer(self, n=1):
        """
        Feed the data buffer by reading a Websocket message.
        :param n: if given, feed buffer until it contains at least n bytes
        """
        buffer = bytearray(self._buffer.read())
        while len(buffer) < n:
            try:
                message = await self._websocket._next_event()
            except anyio.exceptions.ClosedResourceError:
                message = None
            if isinstance(message, CloseConnection):
                message = None
            if message is None:
                self._buffer = None
                break
            if not isinstance(message, BytesMessage):
                raise TypeError("message must be bytes")
            buffer.extend(message.data)
        self._buffer = io.BytesIO(buffer)

    async def write(self, data):
        """
        write some data to the protocol layer
        """
        await self._websocket.send(data)

    def get_peer_info(self):
        sock = self._websocket._sock._socket._raw_socket
        extra_info = sock.getpeername()
        return extra_info[0], extra_info[1]

    async def close(self):
        await self._websocket.close()


class StreamAdapter(BaseAdapter):
    """
    Asyncio Streams API protocol adapter
    This adapter relies on anyio.Stream to read from a TCP socket.
    Because API is very close, this class is trivial
    """
    def __init__(self, stream: anyio.abc.Stream):
        self.logger = logging.getLogger(__name__)
        self._stream = stream

    async def read(self, n=-1) -> bytes:
        if n == -1:
            data = await self._stream.receive_some(4096)
        else:
            data = await self._stream.receive_exactly(n)
        return data

    async def write(self, data):
        await self._stream.send_all(data)

    def get_peer_info(self):
        sock = self._stream._socket._raw_socket
        extra_info = sock.getpeername()
        return extra_info[0], extra_info[1]

    async def close(self):
        await self._stream.close()


class BufferAdapter(BaseAdapter):
    """
    Byte Buffer reader adapter
    This adapter simply adapt reading a byte buffer.
    """
    def __init__(self, buffer: bytes):
        self._rstream = io.BytesIO(buffer)
        self._wstream = io.BytesIO(b'')

    async def read(self, n=-1) -> bytes:
        return self._rstream.read(n)

    async def write(self, data):
        """
        write some data to the protocol layer
        """
        await self._wstream.write(data)

    def get_buffer(self):
        return self._wstream.getvalue()

    def get_peer_info(self):
        return "BufferWriter", 0

    async def close(self):
        self._rstream.close()
        self._wstream.close()



# Copyright (c) 2015 Nicolas JOUANIN
#
# See the file license.txt for copying permission.
from distmqtt.mqtt.packet import MQTTPacket, MQTTFixedHeader, PINGREQ
from distmqtt.errors import DistMQTTException


class PingReqPacket(MQTTPacket):
    VARIABLE_HEADER = None
    PAYLOAD = None

    def __init__(self, fixed: MQTTFixedHeader=None):
        if fixed is None:
            header = MQTTFixedHeader(PINGREQ, 0x00)
        else:
            if fixed.packet_type is not PINGREQ:
                raise DistMQTTException("Invalid fixed packet type %s for PingReqPacket init" % fixed.packet_type)
            header = fixed
        super().__init__(header)
        self.variable_header = None
        self.payload = None



# Copyright (c) 2015 Nicolas JOUANIN
#
# See the file license.txt for copying permission.
import unittest
import anyio
import os
import logging
from distmqtt.client import open_mqttclient, ConnectException
from distmqtt.broker import create_broker
from distmqtt.mqtt.constants import QOS_0, QOS_1, QOS_2

formatter = "[%(asctime)s] %(name)s {%(filename)s:%(lineno)d} %(levelname)s - %(message)s"
logging.basicConfig(level=logging.ERROR, format=formatter)
log = logging.getLogger(__name__)

broker_config = {
    'listeners': {
        'default': {
            'type': 'tcp',
            'bind': '127.0.0.1:1883',
            'max_connections': 10
        },
        'ws': {
            'type': 'ws',
            'bind': '127.0.0.1:8080',
            'max_connections': 10
        },
        'wss': {
            'type': 'ws',
            'bind': '127.0.0.1:8081',
            'max_connections': 10
        },
    },
    'sys_interval': 0,
    'auth': {
        'allow-anonymous': True,
    }
}


class MQTTClientTest(unittest.TestCase):
    def test_connect_tcp(self):
        async def test_coro():
            async with open_mqttclient() as client:
                await client.connect('mqtt://test.mosquitto.org/')
                self.assertIsNotNone(client.session)

        try:
            anyio.run(test_coro)
        except ConnectException:
            log.error("Broken by server")

    def test_connect_tcp_secure(self):
        async def test_coro():
            async with open_mqttclient(config={'check_hostname': False}) as client:
                ca = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'mosquitto.org.crt')
                await client.connect('mqtts://test.mosquitto.org/', cafile=ca)
                self.assertIsNotNone(client.session)

        try:
            anyio.run(test_coro)
        except ConnectException:
            log.error("Broken by server")

    def test_connect_tcp_failure(self):
        async def test_coro():
            try:
                config = {'auto_reconnect': False}
                async with open_mqttclient(config=config) as client:
                    await client.connect('mqtt://127.0.0.1/')
            except ConnectException as e:
                pass
            else:
                raise RuntimeError("should not be able to connect")

        anyio.run(test_coro)

    def test_connect_ws(self):
        async def test_coro():
            async with create_broker(broker_config, plugin_namespace="distmqtt.test.plugins") as broker:
                async with open_mqttclient() as client:
                    await client.connect('ws://127.0.0.1:8080/')
                    self.assertIsNotNone(client.session)

        anyio.run(test_coro, backend="trio")

    def test_reconnect_ws_retain_username_password(self):
        async def test_coro():
            async with create_broker(broker_config, plugin_namespace="distmqtt.test.plugins") as broker:
                async with open_mqttclient() as client:
                    await client.connect('ws://fred:password@127.0.0.1:8080/')
                    self.assertIsNotNone(client.session)
                    await client.reconnect()

                    self.assertIsNotNone(client.session.username)
                    self.assertIsNotNone(client.session.password)

        anyio.run(test_coro, backend="trio")

    def test_connect_ws_secure(self):
        async def test_coro():
            async with create_broker(broker_config, plugin_namespace="distmqtt.test.plugins") as broker:
                async with open_mqttclient() as client:
                    ca = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'mosquitto.org.crt')
                    await client.connect('ws://127.0.0.1:8081/', cafile=ca)
                    self.assertIsNotNone(client.session)

        anyio.run(test_coro, backend="trio")

    def test_ping(self):
        async def test_coro():
            async with create_broker(broker_config, plugin_namespace="distmqtt.test.plugins") as broker:
                async with open_mqttclient() as client:
                    await client.connect('mqtt://127.0.0.1/')
                    self.assertIsNotNone(client.session)
                    await client.ping()

        anyio.run(test_coro, backend="trio")

    def test_subscribe(self):
        async def test_coro():
            async with create_broker(broker_config, plugin_namespace="distmqtt.test.plugins") as broker:
                async with open_mqttclient() as client:
                    await client.connect('mqtt://127.0.0.1/')
                    self.assertIsNotNone(client.session)
                    ret = await client.subscribe([
                        ('$SYS/broker/uptime', QOS_0),
                        ('$SYS/broker/uptime', QOS_1),
                        ('$SYS/broker/uptime', QOS_2),
                    ])
                    self.assertEqual(ret[0], QOS_0)
                    self.assertEqual(ret[1], QOS_1)
                    self.assertEqual(ret[2], QOS_2)

        anyio.run(test_coro, backend="trio")

    def test_unsubscribe(self):
        async def test_coro():
            async with create_broker(broker_config, plugin_namespace="distmqtt.test.plugins") as broker:
                async with open_mqttclient() as client:
                    await client.connect('mqtt://127.0.0.1/')
                    self.assertIsNotNone(client.session)
                    ret = await client.subscribe([
                        ('$SYS/broker/uptime', QOS_0),
                    ])
                    self.assertEqual(ret[0], QOS_0)
                    await client.unsubscribe(['$SYS/broker/uptime'])

        anyio.run(test_coro, backend="trio")

    def test_deliver(self):
        data = b'data'

        async def test_coro():
            async with create_broker(broker_config, plugin_namespace="distmqtt.test.plugins") as broker:
                async with open_mqttclient() as client:
                    await client.connect('mqtt://127.0.0.1/')
                    self.assertIsNotNone(client.session)
                    ret = await client.subscribe([
                        ('test_topic', QOS_0),
                    ])
                    self.assertEqual(ret[0], QOS_0)
                    async with open_mqttclient() as client_pub:
                        await client_pub.connect('mqtt://127.0.0.1/')
                        await client_pub.publish('test_topic', data, QOS_0)
                    message = await client.deliver_message()
                    self.assertIsNotNone(message)
                    self.assertIsNotNone(message.publish_packet)
                    self.assertEqual(message.data, data)
                    await client.unsubscribe(['$SYS/broker/uptime'])

        anyio.run(test_coro, backend="trio")

    def test_deliver_timeout(self):
        async def test_coro():
            async with create_broker(broker_config, plugin_namespace="distmqtt.test.plugins") as broker:
                async with open_mqttclient() as client:
                    await client.connect('mqtt://127.0.0.1/')
                    self.assertIsNotNone(client.session)
                    ret = await client.subscribe([
                        ('test_topic', QOS_0),
                    ])
                    self.assertEqual(ret[0], QOS_0)
                    with self.assertRaises(TimeoutError):
                        async with anyio.fail_after(2):
                            await client.deliver_message()
                    await client.unsubscribe(['$SYS/broker/uptime'])

        anyio.run(test_coro, backend="trio")



import logging
import anyio

from distmqtt.client import open_mqttclient


#
# This sample shows a client running idle.
# Meanwhile, keepalive is managed through PING messages sent every 5 seconds
#


logger = logging.getLogger(__name__)

config = {
    'keep_alive': 5,
    'ping_delay': 1,
}


async def test_coro():
    async with open_mqttclient() as C:
        await C.connect('mqtt://test.mosquitto.org:1883/')
        await anyio.sleep(18)


if __name__ == '__main__':
    formatter = "[%(asctime)s] {%(filename)s:%(lineno)d} %(levelname)s - %(message)s"
    logging.basicConfig(level=logging.DEBUG, format=formatter)
    anyio.run(test_coro)



# Copyright (c) 2015 Nicolas JOUANIN
#
# See the file license.txt for copying permission.
import anyio
import unittest

from distmqtt.mqtt.subscribe import SubscribePacket, SubscribePayload
from distmqtt.mqtt.packet import PacketIdVariableHeader
from distmqtt.mqtt.constants import QOS_1, QOS_2
from distmqtt.adapters import BufferAdapter


class SubscribePacketTest(unittest.TestCase):
    def test_from_stream(self):
        data = b'\x80\x0e\x00\x0a\x00\x03a/b\x01\x00\x03c/d\x02'
        stream = BufferAdapter(data)
        message = anyio.run(SubscribePacket.from_stream, stream)
        (topic, qos) = message.payload.topics[0]
        self.assertEqual(topic, 'a/b')
        self.assertEqual(qos, QOS_1)
        (topic, qos) = message.payload.topics[1]
        self.assertEqual(topic, 'c/d')
        self.assertEqual(qos, QOS_2)

    def test_to_stream(self):
        variable_header = PacketIdVariableHeader(10)
        payload = SubscribePayload(
            [
                ('a/b', QOS_1),
                ('c/d', QOS_2)
            ])
        publish = SubscribePacket(variable_header=variable_header, payload=payload)
        out = publish.to_bytes()
        self.assertEqual(out, b'\x82\x0e\x00\x0a\x00\x03a/b\x01\x00\x03c/d\x02')



# Copyright (c) 2015 Nicolas JOUANIN
#
# See the file license.txt for copying permission.
"""
DistMQTT - MQTT 3.1.1 broker

Usage:
    distmqtt --help
"""

import sys
import logging
import anyio
import os
import asyncclick as click
from distmqtt.broker import create_broker
from distmqtt.utils import read_yaml_config


default_config = {
    'listeners': {
        'default': {
            'type': 'tcp',
            'bind': '0.0.0.0:1883',
        },
    },
    'sys_interval': 10,
    'auth': {
        'allow-anonymous': True,
        'password-file': os.path.join(os.path.dirname(os.path.realpath(__file__)), "passwd"),
        'plugins': [
            'auth_file', 'auth_anonymous'
        ]
    },
    'topic-check': {
        'enabled': False
    }
}

logger = logging.getLogger(__name__)


@click.command()
@click.version_option()
@click.option("-c","--config", help="Name of config file (YAML)")
@click.option("-d","--debug", is_flag=True, help="Debug?")
async def main(config, debug):
    formatter = "[%(asctime)s] :: %(levelname)s - %(message)s"

    level = logging.DEBUG if debug else logging.INFO
    logging.basicConfig(level=level, format=formatter)

    if not config:
        config = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'default_broker.yaml')
        logger.debug("Using default configuration")
    config = read_yaml_config(config)

    async with create_broker(config) as broker:
        while True:
            await anyio.sleep(99999)

if __name__ == "__main__":
    main()



# Copyright (c) 2015 Nicolas JOUANIN
#
# See the file license.txt for copying permission.
import anyio
import unittest

from distmqtt.mqtt.publish import PublishPacket, PublishVariableHeader, PublishPayload
from distmqtt.adapters import BufferAdapter
from distmqtt.mqtt.constants import QOS_0, QOS_1, QOS_2


class PublishPacketTest(unittest.TestCase):
    def test_from_stream_qos_0(self):
        data = b'\x31\x11\x00\x05topic0123456789'
        stream = BufferAdapter(data)
        message = anyio.run(PublishPacket.from_stream, stream)
        self.assertEqual(message.variable_header.topic_name, 'topic')
        self.assertEqual(message.variable_header.packet_id, None)
        self.assertFalse((message.fixed_header.flags >> 1) & 0x03)
        self.assertTrue(message.fixed_header.flags & 0x01)
        self.assertTrue(message.payload.data, b'0123456789')

    def test_from_stream_qos_2(self):
        data = b'\x37\x13\x00\x05topic\x00\x0a0123456789'
        stream = BufferAdapter(data)
        message = anyio.run(PublishPacket.from_stream, stream)
        self.assertEqual(message.variable_header.topic_name, 'topic')
        self.assertEqual(message.variable_header.packet_id, 10)
        self.assertTrue((message.fixed_header.flags >> 1) & 0x03)
        self.assertTrue(message.fixed_header.flags & 0x01)
        self.assertTrue(message.payload.data, b'0123456789')

    def test_to_stream_no_packet_id(self):
        variable_header = PublishVariableHeader('topic', None)
        payload = PublishPayload(b'0123456789')
        publish = PublishPacket(variable_header=variable_header, payload=payload)
        out = publish.to_bytes()
        self.assertEqual(out, b'\x30\x11\x00\x05topic0123456789')

    def test_to_stream_packet(self):
        variable_header = PublishVariableHeader('topic', 10)
        payload = PublishPayload(b'0123456789')
        publish = PublishPacket(variable_header=variable_header, payload=payload)
        out = publish.to_bytes()
        self.assertEqual(out, b'\x30\x13\x00\x05topic\00\x0a0123456789')

    def test_build(self):
        packet = PublishPacket.build('/topic', b'data', 1, False, QOS_0, False)
        self.assertEqual(packet.packet_id, 1)
        self.assertFalse(packet.dup_flag)
        self.assertEqual(packet.qos, QOS_0)
        self.assertFalse(packet.retain_flag)

        packet = PublishPacket.build('/topic', b'data', 1, False, QOS_1, False)
        self.assertEqual(packet.packet_id, 1)
        self.assertFalse(packet.dup_flag)
        self.assertEqual(packet.qos, QOS_1)
        self.assertFalse(packet.retain_flag)

        packet = PublishPacket.build('/topic', b'data', 1, False, QOS_2, False)
        self.assertEqual(packet.packet_id, 1)
        self.assertFalse(packet.dup_flag)
        self.assertEqual(packet.qos, QOS_2)
        self.assertFalse(packet.retain_flag)

        packet = PublishPacket.build('/topic', b'data', 1, True, QOS_0, False)
        self.assertEqual(packet.packet_id, 1)
        self.assertTrue(packet.dup_flag)
        self.assertEqual(packet.qos, QOS_0)
        self.assertFalse(packet.retain_flag)

        packet = PublishPacket.build('/topic', b'data', 1, True, QOS_1, False)
        self.assertEqual(packet.packet_id, 1)
        self.assertTrue(packet.dup_flag)
        self.assertEqual(packet.qos, QOS_1)
        self.assertFalse(packet.retain_flag)

        packet = PublishPacket.build('/topic', b'data', 1, True, QOS_2, False)
        self.assertEqual(packet.packet_id, 1)
        self.assertTrue(packet.dup_flag)
        self.assertEqual(packet.qos, QOS_2)
        self.assertFalse(packet.retain_flag)

        packet = PublishPacket.build('/topic', b'data', 1, False, QOS_0, True)
        self.assertEqual(packet.packet_id, 1)
        self.assertFalse(packet.dup_flag)
        self.assertEqual(packet.qos, QOS_0)
        self.assertTrue(packet.retain_flag)

        packet = PublishPacket.build('/topic', b'data', 1, False, QOS_1, True)
        self.assertEqual(packet.packet_id, 1)
        self.assertFalse(packet.dup_flag)
        self.assertEqual(packet.qos, QOS_1)
        self.assertTrue(packet.retain_flag)

        packet = PublishPacket.build('/topic', b'data', 1, False, QOS_2, True)
        self.assertEqual(packet.packet_id, 1)
        self.assertFalse(packet.dup_flag)
        self.assertEqual(packet.qos, QOS_2)
        self.assertTrue(packet.retain_flag)

        packet = PublishPacket.build('/topic', b'data', 1, True, QOS_0, True)
        self.assertEqual(packet.packet_id, 1)
        self.assertTrue(packet.dup_flag)
        self.assertEqual(packet.qos, QOS_0)
        self.assertTrue(packet.retain_flag)

        packet = PublishPacket.build('/topic', b'data', 1, True, QOS_1, True)
        self.assertEqual(packet.packet_id, 1)
        self.assertTrue(packet.dup_flag)
        self.assertEqual(packet.qos, QOS_1)
        self.assertTrue(packet.retain_flag)

        packet = PublishPacket.build('/topic', b'data', 1, True, QOS_2, True)
        self.assertEqual(packet.packet_id, 1)
        self.assertTrue(packet.dup_flag)
        self.assertEqual(packet.qos, QOS_2)
        self.assertTrue(packet.retain_flag)



# Copyright (c) 2015 Nicolas JOUANIN
#
# See the file license.txt for copying permission.
import anyio
import unittest

from distmqtt.mqtt.unsubscribe import UnsubscribePacket, UnubscribePayload
from distmqtt.mqtt.packet import PacketIdVariableHeader
from distmqtt.adapters import BufferAdapter


class UnsubscribePacketTest(unittest.TestCase):
    def test_from_stream(self):
        data = b'\xa2\x0c\x00\n\x00\x03a/b\x00\x03c/d'
        stream = BufferAdapter(data)
        message = anyio.run(UnsubscribePacket.from_stream, stream)
        self.assertEqual(message.payload.topics[0], 'a/b')
        self.assertEqual(message.payload.topics[1], 'c/d')

    def test_to_stream(self):
        variable_header = PacketIdVariableHeader(10)
        payload = UnubscribePayload(['a/b', 'c/d'])
        publish = UnsubscribePacket(variable_header=variable_header, payload=payload)
        out = publish.to_bytes()
        self.assertEqual(out, b'\xa2\x0c\x00\n\x00\x03a/b\x00\x03c/d')



# Copyright (c) 2015 Nicolas JOUANIN
#
# See the file license.txt for copying permission.

import unittest
import logging
import os
import anyio
from distmqtt.plugins.manager import BaseContext
from distmqtt.plugins.authentication import AnonymousAuthPlugin, FileAuthPlugin
from distmqtt.session import Session

formatter = "[%(asctime)s] %(name)s {%(filename)s:%(lineno)d} %(levelname)s - %(message)s"
logging.basicConfig(level=logging.DEBUG, format=formatter)


class TestAnonymousAuthPlugin(unittest.TestCase):
    def test_allow_anonymous(self):
        context = BaseContext()
        context.logger = logging.getLogger(__name__)
        context.config = {
            'auth': {
                'allow-anonymous': True
            }
        }
        async def coro():
            s = Session(None)
            s.username = ""
            auth_plugin = AnonymousAuthPlugin(context)
            ret = await auth_plugin.authenticate(session=s)
            self.assertTrue(ret)
        anyio.run(coro)

    def test_disallow_anonymous(self):
        context = BaseContext()
        context.logger = logging.getLogger(__name__)
        context.config = {
            'auth': {
                'allow-anonymous': False
            }
        }
        async def coro():
            s = Session(None)
            s.username = ""
            auth_plugin = AnonymousAuthPlugin(context)
            ret = await auth_plugin.authenticate(session=s)
            self.assertFalse(ret)
        anyio.run(coro)

    def test_allow_nonanonymous(self):
        context = BaseContext()
        context.logger = logging.getLogger(__name__)
        context.config = {
            'auth': {
                'allow-anonymous': False
            }
        }
        async def coro():
            s = Session(None)
            s.username = "test"
            auth_plugin = AnonymousAuthPlugin(context)
            ret = await auth_plugin.authenticate(session=s)
            self.assertTrue(ret)
        anyio.run(coro)


class TestFileAuthPlugin(unittest.TestCase):
    def test_allow(self):
        context = BaseContext()
        context.logger = logging.getLogger(__name__)
        context.config = {
            'auth': {
                'password-file': os.path.join(os.path.dirname(os.path.realpath(__file__)), "passwd")
            }
        }
        async def coro():
            s = Session(None)
            s.username = "user"
            s.password = "test"
            auth_plugin = FileAuthPlugin(context)
            ret = await auth_plugin.authenticate(session=s)
            self.assertTrue(ret)
        anyio.run(coro)

    def test_wrong_password(self):
        context = BaseContext()
        context.logger = logging.getLogger(__name__)
        context.config = {
            'auth': {
                'password-file': os.path.join(os.path.dirname(os.path.realpath(__file__)), "passwd")
            }
        }
        async def coro():
            s = Session(None)
            s.username = "user"
            s.password = "wrong password"
            auth_plugin = FileAuthPlugin(context)
            ret = await auth_plugin.authenticate(session=s)
            self.assertFalse(ret)
        anyio.run(coro)

    def test_unknown_password(self):
        context = BaseContext()
        context.logger = logging.getLogger(__name__)
        context.config = {
            'auth': {
                'password-file': os.path.join(os.path.dirname(os.path.realpath(__file__)), "passwd")
            }
        }
        async def coro():
            s = Session(None)
            s.username = "some user"
            s.password = "some password"
            auth_plugin = FileAuthPlugin(context)
            ret = await auth_plugin.authenticate(session=s)
            self.assertFalse(ret)
        anyio.run(coro)



# Copyright (c) 2015 Nicolas JOUANIN
#
# See the file license.txt for copying permission.
"""
distmqtt_sub - MQTT 3.1.1 publisher

Usage:
    distmqtt_sub --version
    distmqtt_sub (-h | --help)
    distmqtt_sub --url BROKER_URL -t TOPIC... [-n COUNT] [-c CONFIG_FILE] [-C codec] [-i CLIENT_ID] [-q | --qos QOS] [-d] [-k KEEP_ALIVE] [--clean-session] [--ca-file CAFILE] [--ca-path CAPATH] [--ca-data CADATA] [ --will-topic WILL_TOPIC [--will-message WILL_MESSAGE] [--will-qos WILL_QOS] [--will-retain] ] [--extra-headers HEADER]

Options:
    -h --help           Show this screen.
    --version           Show version.
    --url BROKER_URL    Broker connection URL (musr conform to MQTT URI scheme (see https://github.com/mqtt/mqtt.github.io/wiki/URI-Scheme>)
    -c CONFIG_FILE      Broker configuration file (YAML format)
    -i CLIENT_ID        Id to use as client ID.
    -n COUNT            Number of messages to read before ending.
    -q | --qos QOS      Quality of service desired to receive messages, from 0, 1 and 2. Defaults to 0.
    -t TOPIC...         Topic filter to subcribe
    -k KEEP_ALIVE       Keep alive timeout in second
    -C codec            use this codec
    --clean-session     Clean session on connect (defaults to False)
    --ca-file CAFILE]   CA file
    --ca-path CAPATH]   CA Path
    --ca-data CADATA    CA data
    --will-topic WILL_TOPIC
    --will-message WILL_MESSAGE
    --will-qos WILL_QOS
    --will-retain
    --extra-headers EXTRA_HEADERS      JSON object with key-value pairs of additional headers for websocket connections
    -d                  Enable debug messages
"""

import sys
import logging
import anyio
import os
import json
from distmqtt.client import open_mqttclient, ConnectException
from distmqtt.errors import MQTTException
from distmqtt.version import get_version
from docopt import docopt
from distmqtt.mqtt.constants import QOS_0
from distmqtt.utils import read_yaml_config

logger = logging.getLogger(__name__)


def _gen_client_id():
    import os
    import socket
    pid = os.getpid()
    hostname = socket.gethostname()
    return "distmqtt_sub/%d-%s" % (pid, hostname)


def _get_qos(arguments):
    try:
        return int(arguments['--qos'][0])
    except:
        return QOS_0

def _get_extra_headers(arguments):
    try:
        return json.loads(arguments['--extra-headers'])
    except:
        return {}


async def do_sub(client, arguments):

    try:
        await client.connect(uri=arguments['--url'],
                                  cleansession=arguments['--clean-session'],
                                  cafile=arguments['--ca-file'],
                                  capath=arguments['--ca-path'],
                                  cadata=arguments['--ca-data'],
                                  extra_headers=_get_extra_headers(arguments))
        filters = []
        async with anyio.create_task_group() as tg:
            for topic in arguments['-t']:
                await tg.spawn(run_sub,client,topic, arguments)

    except KeyboardInterrupt:
        pass
    except ConnectException as ce:
        logger.fatal("connection to '%s' failed: %r", arguments['--url'], ce)
    finally:
        await client.disconnect()

async def run_sub(client,topic, arguments):
    qos = _get_qos(arguments)
    if arguments['-n']:
        max_count = int(arguments['-n'])
    else:
        max_count = None
    count = 0

    async with client.subscription(topic, qos) as sub:
        async for message in sub:
            count += 1
            print(message.data)
            if max_count and count >= max_count:
                break


async def main(*args, **kwargs):
    arguments = docopt(__doc__, version=get_version())
    #print(arguments)
    formatter = "[%(asctime)s] :: %(levelname)s - %(message)s"

    if arguments['-d']:
        level = logging.DEBUG
    else:
        level = logging.INFO
    logging.basicConfig(level=level, format=formatter)

    config = None
    if arguments['-c']:
        config = read_yaml_config(arguments['-c'])
    else:
        config = read_yaml_config(os.path.join(os.path.dirname(os.path.realpath(__file__)), 'default_client.yaml'))
        logger.debug(os.path.join(os.path.dirname(os.path.realpath(__file__)), 'default_client.yaml'))
        logger.debug("Using default configuration")

    client_id = arguments.get("-i", None)
    if not client_id:
        client_id = _gen_client_id()

    if arguments['-k']:
        config['keep_alive'] = int(arguments['-k'])

    if arguments['--will-topic'] and arguments['--will-message'] and arguments['--will-qos']:
        config['will'] = dict()
        config['will']['topic'] = arguments['--will-topic']
        config['will']['message'] = arguments['--will-message'].encode('utf-8')
        config['will']['qos'] = int(arguments['--will-qos'])
        config['will']['retain'] = arguments['--will-retain']

    async with open_mqttclient(client_id=client_id, config=config, codec=arguments['-C']) as C:
        await do_sub(C, arguments)


if __name__ == "__main__":
    anyio.run(main)



import logging
import anyio

from distmqtt.client import open_mqttclient, ConnectException


#
# This sample shows how to publish messages to broker using different QOS
# Debug outputs shows the message flows
#

logger = logging.getLogger(__name__)


async def test_coro():
    try:
        async with open_mqttclient() as C:
            await C.connect('mqtt://0.0.0.0:1883')

            await C.publish('data/classified', b'TOP SECRET', qos=0x01)
            await C.publish('data/memes', b'REAL FUN', qos=0x01)
            await C.publish('repositories/distmqtt/master', b'NEW STABLE RELEASE', qos=0x01)
            await C.publish('repositories/distmqtt/devel', b'THIS NEEDS TO BE CHECKED', qos=0x01)
            await C.publish('calendar/distmqtt/releases', b'NEW RELEASE', qos=0x01)
            logger.info("messages published")
    except ConnectException as ce:
        logger.error("Connection failed: %r", ce)


if __name__ == '__main__':
    formatter = "[%(asctime)s] %(name)s {%(filename)s:%(lineno)d} %(levelname)s - %(message)s"
    formatter = "%(message)s"
    logging.basicConfig(level=logging.DEBUG, format=formatter)
    anyio.run(test_coro)



# Copyright (c) 2015 Nicolas JOUANIN
#
# See the file license.txt for copying permission.
import unittest
import anyio
import logging
import random
from distmqtt.plugins.manager import PluginManager
from distmqtt.session import Session, OutgoingApplicationMessage, IncomingApplicationMessage
from distmqtt.mqtt.protocol.handler import ProtocolHandler
from distmqtt.adapters import StreamAdapter
from distmqtt.mqtt.constants import QOS_0, QOS_1, QOS_2
from distmqtt.mqtt.publish import PublishPacket
from distmqtt.mqtt.puback import PubackPacket
from distmqtt.mqtt.pubrec import PubrecPacket
from distmqtt.mqtt.pubrel import PubrelPacket
from distmqtt.mqtt.pubcomp import PubcompPacket

formatter = "[%(asctime)s] %(name)s {%(filename)s:%(lineno)d} %(levelname)s - %(message)s"
logging.basicConfig(level=logging.DEBUG, format=formatter)
log = logging.getLogger(__name__)


def rand_packet_id():
    return random.randint(0, 65535)


def adapt(conn):
    return StreamAdapter(conn)


class ProtocolHandlerTest(unittest.TestCase):
    async def listen_(self, server_mock, server):
        while True:
            sock = await server.accept()
            server_mock(sock, sock)

    def run_(self, server_mock, test_coro):
        async def runner():
            async with anyio.create_task_group() as tg:
                self.plugin_manager = PluginManager(tg, "distmqtt.test.plugins", context=None)
                async with await anyio.create_tcp_server(port=8888, interface="127.0.0.1") as server:
                    await tg.spawn(self.listen_, server_mock, server)
                    async with await anyio.connect_tcp("127.0.0.1", server.port) as conn:
                        sr = adapt(conn)
                        await test_coro(sr)
            anyio.run(runner)

    def test_start_stop(self):
        async def server_mock(stream):
            pass

        async def test_coro(stream_adapted):
            try:
                s = Session(None)
                handler = ProtocolHandler(self.plugin_manager)
                handler.attach(s, stream_adapted)
                await self.start_handler(handler, s)
                await self.stop_handler(handler, s)
            except Exception as ae:
                raise

        self.run_(server_mock, test_coro)

    def test_publish_qos0(self):
        async def server_mock(stream):
            try:
                packet = await PublishPacket.from_stream(stream)
                self.assertEqual(packet.variable_header.topic_name, '/topic')
                self.assertEqual(packet.qos, QOS_0)
                self.assertIsNone(packet.packet_id)
            except Exception as ae:
                raise

        async def test_coro(stream_adapted):
            try:
                s = Session(None)
                handler = ProtocolHandler(self.plugin_manager)
                handler.attach(s, stream_adapted)
                await self.start_handler(handler, s)
                message = await handler.mqtt_publish('/topic', b'test_data', QOS_0, False)
                self.assertIsInstance(message, OutgoingApplicationMessage)
                self.assertIsNotNone(message.publish_packet)
                self.assertIsNone(message.puback_packet)
                self.assertIsNone(message.pubrec_packet)
                self.assertIsNone(message.pubrel_packet)
                self.assertIsNone(message.pubcomp_packet)
                await self.stop_handler(handler, s)
            except Exception as ae:
                raise

        self.run_(server_mock, test_coro)

    def test_publish_qos1(self):
        async def server_mock(stream):
            packet = await PublishPacket.from_stream(stream)
            try:
                self.assertEqual(packet.variable_header.topic_name, '/topic')
                self.assertEqual(packet.qos, QOS_1)
                self.assertIsNotNone(packet.packet_id)
                self.assertIn(packet.packet_id, self.session.inflight_out)
                self.assertIn(packet.packet_id, self.handler._puback_waiters)
                puback = PubackPacket.build(packet.packet_id)
                await puback.to_stream(stream)
            except Exception as ae:
                raise

        async def test_coro(stream_adapted):
            try:
                self.session = Session(None)
                self.handler = ProtocolHandler(self.plugin_manager)
                self.handler.attach(self.session, stream_adapted)
                await self.start_handler(self.handler, self.session)
                message = await self.handler.mqtt_publish('/topic', b'test_data', QOS_1, False)
                self.assertIsInstance(message, OutgoingApplicationMessage)
                self.assertIsNotNone(message.publish_packet)
                self.assertIsNotNone(message.puback_packet)
                self.assertIsNone(message.pubrec_packet)
                self.assertIsNone(message.pubrel_packet)
                self.assertIsNone(message.pubcomp_packet)
                await self.stop_handler(self.handler, self.session)
            except Exception as ae:
                raise
        self.handler = None
        self.run_(server_mock, test_coro)

    def test_publish_qos2(self):
        async def server_mock(stream):
            try:
                packet = await PublishPacket.from_stream(stream)
                self.assertEqual(packet.topic_name, '/topic')
                self.assertEqual(packet.qos, QOS_2)
                self.assertIsNotNone(packet.packet_id)
                self.assertIn(packet.packet_id, self.session.inflight_out)
                self.assertIn(packet.packet_id, self.handler._pubrec_waiters)
                pubrec = PubrecPacket.build(packet.packet_id)
                await pubrec.to_stream(stream)

                await PubrelPacket.from_stream(stream)
                self.assertIn(packet.packet_id, self.handler._pubcomp_waiters)
                pubcomp = PubcompPacket.build(packet.packet_id)
                await pubcomp.to_stream(stream)
            except Exception as ae:
                raise

        async def test_coro(stream_adapted):
            self.session = Session(None)
            try:
                self.handler = ProtocolHandler(self.plugin_manager)
                self.handler.attach(self.session, stream_adapted)
                await self.start_handler(self.handler, self.session)
                message = await self.handler.mqtt_publish('/topic', b'test_data', QOS_2, False)
                self.assertIsInstance(message, OutgoingApplicationMessage)
                self.assertIsNotNone(message.publish_packet)
                self.assertIsNone(message.puback_packet)
                self.assertIsNotNone(message.pubrec_packet)
                self.assertIsNotNone(message.pubrel_packet)
                self.assertIsNotNone(message.pubcomp_packet)
                await self.stop_handler(self.handler, self.session)
            except Exception as ae:
                raise
        self.handler = None

        self.run_(server_mock, test_coro)

    def test_receive_qos0(self):
        async def server_mock(stream):
            packet = PublishPacket.build('/topic', b'test_data', rand_packet_id(), False, QOS_0, False)
            await packet.to_stream(stream)

        async def test_coro(stream_adapted):
            self.session = Session(None)
            try:
                self.handler = ProtocolHandler(self.plugin_manager)
                self.handler.attach(self.session, stream_adapted)
                await self.start_handler(self.handler, self.session)
                message = await self.handler.mqtt_deliver_next_message()
                self.assertIsInstance(message, IncomingApplicationMessage)
                self.assertIsNotNone(message.publish_packet)
                self.assertIsNone(message.puback_packet)
                self.assertIsNone(message.pubrec_packet)
                self.assertIsNone(message.pubrel_packet)
                self.assertIsNone(message.pubcomp_packet)
                await self.stop_handler(self.handler, self.session)
            except Exception as ae:
                raise

        self.handler = None
        self.run_(server_mock, test_coro)

    def test_receive_qos1(self):
        async def server_mock(stream):
            try:
                packet = PublishPacket.build('/topic', b'test_data', rand_packet_id(), False, QOS_1, False)
                await packet.to_stream(stream)
                puback = await PubackPacket.from_stream(stream)
                self.assertIsNotNone(puback)
                self.assertEqual(packet.packet_id, puback.packet_id)
            except Exception as ae:
                print(ae)
                raise

        async def test_coro(stream_adapted):
            self.session = Session(None)
            try:
                self.handler = ProtocolHandler(self.plugin_manager)
                self.handler.attach(self.session, stream_adapted)
                await self.start_handler(self.handler, self.session)
                message = await self.handler.mqtt_deliver_next_message()
                self.assertIsInstance(message, IncomingApplicationMessage)
                self.assertIsNotNone(message.publish_packet)
                self.assertIsNotNone(message.puback_packet)
                self.assertIsNone(message.pubrec_packet)
                self.assertIsNone(message.pubrel_packet)
                self.assertIsNone(message.pubcomp_packet)
                await self.stop_handler(self.handler, self.session)
            except Exception as ae:
                raise

        self.handler = None
        self.run_(server_mock, test_coro)

    def test_receive_qos2(self):
        async def server_mock(stream):
            try:
                packet = PublishPacket.build('/topic', b'test_data', rand_packet_id(), False, QOS_2, False)
                await packet.to_stream(stream)
                pubrec = await PubrecPacket.from_stream(stream)
                self.assertIsNotNone(pubrec)
                self.assertEqual(packet.packet_id, pubrec.packet_id)
                self.assertIn(packet.packet_id, self.handler._pubrel_waiters)
                pubrel = PubrelPacket.build(packet.packet_id)
                await pubrel.to_stream(stream)
                pubcomp = await PubcompPacket.from_stream(stream)
                self.assertIsNotNone(pubcomp)
                self.assertEqual(packet.packet_id, pubcomp.packet_id)
            except Exception as ae:
                raise

        async def test_coro(stream_adapted):
            self.session = Session(None)
            try:
                self.handler = ProtocolHandler(self.plugin_manager)
                self.handler.attach(self.session, stream_adapted)
                await self.start_handler(self.handler, self.session)
                message = await self.handler.mqtt_deliver_next_message()
                self.assertIsInstance(message, IncomingApplicationMessage)
                self.assertIsNotNone(message.publish_packet)
                self.assertIsNone(message.puback_packet)
                self.assertIsNotNone(message.pubrec_packet)
                self.assertIsNotNone(message.pubrel_packet)
                self.assertIsNotNone(message.pubcomp_packet)
                await self.stop_handler(self.handler, self.session)
            except Exception as ae:
                raise

        self.handler = None
        self.run_(server_mock, test_coro)

    async def start_handler(self, handler, session):
        self.check_empty_waiters(handler)
        self.check_no_message(session)
        await handler.start()
        self.assertTrue(handler._reader_ready)

    async def stop_handler(self, handler, session):
        await handler.stop()
        self.assertTrue(handler._reader_stopped)
        self.check_empty_waiters(handler)
        self.check_no_message(session)

    def check_empty_waiters(self, handler):
        self.assertFalse(handler._puback_waiters)
        self.assertFalse(handler._pubrec_waiters)
        self.assertFalse(handler._pubrel_waiters)
        self.assertFalse(handler._pubcomp_waiters)

    def check_no_message(self, session):
        self.assertFalse(session.inflight_out)
        self.assertFalse(session.inflight_in)

    def test_publish_qos1_retry(self):
        async def server_mock(stream):
            packet = await PublishPacket.from_stream(stream)
            try:
                self.assertEqual(packet.topic_name, '/topic')
                self.assertEqual(packet.qos, QOS_1)
                self.assertIsNotNone(packet.packet_id)
                self.assertIn(packet.packet_id, self.session.inflight_out)
                self.assertIn(packet.packet_id, self.handler._puback_waiters)
                puback = PubackPacket.build(packet.packet_id)
                await puback.to_stream(stream)
            except Exception as ae:
                raise

        async def test_coro(stream_adapted):
            self.session = Session(None)
            message = OutgoingApplicationMessage(1, '/topic', QOS_1, b'test_data', False)
            message.publish_packet = PublishPacket.build('/topic', b'test_data', rand_packet_id(), False, QOS_1, False)
            self.session.inflight_out[1] = message
            try:
                self.handler = ProtocolHandler(self.plugin_manager)
                self.handler.attach(self.session, stream_adapted)
                await self.handler.start()
                await self.stop_handler(self.handler, self.session)
            except Exception as ae:
                raise
        self.handler = None

        self.run_(server_mock, test_coro)

    def test_publish_qos2_retry(self):
        async def server_mock(stream):
            try:
                packet = await PublishPacket.from_stream(stream)
                self.assertEqual(packet.topic_name, '/topic')
                self.assertEqual(packet.qos, QOS_2)
                self.assertIsNotNone(packet.packet_id)
                self.assertIn(packet.packet_id, self.session.inflight_out)
                self.assertIn(packet.packet_id, self.handler._pubrec_waiters)
                pubrec = PubrecPacket.build(packet.packet_id)
                await pubrec.to_stream(stream)

                await PubrelPacket.from_stream(stream)
                self.assertIn(packet.packet_id, self.handler._pubcomp_waiters)
                pubcomp = PubcompPacket.build(packet.packet_id)
                await pubcomp.to_stream(stream)
            except Exception as ae:
                raise

        async def test_coro(stream_adapted):
            self.session = Session(None)
            message = OutgoingApplicationMessage(1, '/topic', QOS_2, b'test_data', False)
            message.publish_packet = PublishPacket.build('/topic', b'test_data', rand_packet_id(), False, QOS_2, False)
            self.session.inflight_out[1] = message
            try:
                self.handler = ProtocolHandler(self.plugin_manager)
                self.handler.attach(self.session, stream_adapted)
                await self.handler.start()
                await self.stop_handler(self.handler, self.session)
            except Exception as ae:
                raise
        self.handler = None

        self.run_(server_mock, test_coro)



# Copyright (c) 2015 Nicolas JOUANIN
#
# See the file license.txt for copying permission.

from setuptools import setup, find_packages

setup(
    name="distmqtt",
    use_scm_version={"version_scheme": "guess-next-dev", "local_scheme": "dirty-tag"},
    setup_requires=["setuptools_scm", "pytest-runner"],
    description="MQTT client/broker using anyio and distkv",
    author="Matthias Urlichs",
    author_email='matthias@urlichs.de',
    url="https://github.com/smurfix/distmqtt",
    license='MIT',
    packages=find_packages(exclude=['tests']),
    include_package_data=True,
    platforms='all',
    python_requires=">=3.6",
    install_requires=[
        'transitions',
        'asyncwebsockets',
        'passlib',
        'docopt',
        'pyyaml',
        'anyio',
        'attrs',
        'distkv >= 0.7.0',
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: POSIX',
        'Operating System :: MacOS',
        'Operating System :: Microsoft :: Windows',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Communications',
        'Topic :: Internet'
    ],
    entry_points={
        'distmqtt.test.plugins': [
            'test_plugin = tests.plugins.test_manager:SimpleTestPlugin',
            'event_plugin = tests.plugins.test_manager:EventTestPlugin',
            'packet_logger_plugin = distmqtt.plugins.logging:PacketLoggerPlugin',
        ],
        'distmqtt.broker.plugins': [
            # 'event_logger_plugin = distmqtt.plugins.logging:EventLoggerPlugin',
            'packet_logger_plugin = distmqtt.plugins.logging:PacketLoggerPlugin',
            'auth_anonymous = distmqtt.plugins.authentication:AnonymousAuthPlugin',
            'auth_file = distmqtt.plugins.authentication:FileAuthPlugin',
            'topic_taboo = distmqtt.plugins.topic_checking:TopicTabooPlugin',
            'topic_acl = distmqtt.plugins.topic_checking:TopicAccessControlListPlugin',
            'broker_sys = distmqtt.plugins.sys.broker:BrokerSysPlugin',
        ],
        'distmqtt.client.plugins': [
            'packet_logger_plugin = distmqtt.plugins.logging:PacketLoggerPlugin',
        ],
        'console_scripts': [
            'distmqtt = distmqtt.scripts.broker_script:main',
            'distmqtt_pub = distmqtt.scripts.pub_script:main',
            'distmqtt_sub = distmqtt.scripts.sub_script:main',
        ]
    }
)



import logging
import anyio
from functools import partial

from distmqtt.client import open_mqttclient, ConnectException
from distmqtt.mqtt.constants import QOS_0, QOS_1, QOS_2


#
# This sample shows how to publish messages to broker using different QOS
# Debug outputs shows the message flows
#

logger = logging.getLogger(__name__)

config = {}


async def test_coro():
    async with open_mqttclient(config=config) as sub_client:
        await sub_client.connect('mqtt://127.0.0.1', cleansession=False)
        ret = await sub_client.subscribe([('/qos0', QOS_0), ('/qos1', QOS_1), ('/qos2', QOS_2)])
        await sub_client.disconnect()
        await anyio.sleep(0.1)

        await _client_publish('/qos0', b'data', QOS_0, retain=True)
        await _client_publish('/qos1', b'data', QOS_1, retain=True)
        await _client_publish('/qos2', b'data', QOS_2, retain=True)
        import pdb;pdb.set_trace()
        await sub_client.reconnect()
        for qos in [QOS_0, QOS_1, QOS_2]:
            logger.debug("TEST QOS: %d", qos)
            message = await sub_client.deliver_message()
            logger.debug("Message: " + repr(message.publish_packet))
    await anyio.sleep(0.1)


async def _client_publish(topic, data, qos, retain=False):
    async with open_mqttclient() as pub_client:
        ret = await pub_client.connect('mqtt://127.0.0.1/')
        ret = await pub_client.publish(topic, data, qos, retain)
    return ret

if __name__ == '__main__':
    formatter = "[%(asctime)s] %(name)s {%(filename)s:%(lineno)d} %(levelname)s - %(message)s"
    #formatter = "%(message)s"
    logging.basicConfig(level=logging.DEBUG, format=formatter)
    anyio.run(test_coro)



# Copyright (c) 2015 Nicolas JOUANIN
#
# See the file license.txt for copying permission.
from distmqtt.mqtt.packet import MQTTPacket, MQTTFixedHeader, UNSUBACK, PacketIdVariableHeader
from distmqtt.errors import DistMQTTException


class UnsubackPacket(MQTTPacket):
    VARIABLE_HEADER = PacketIdVariableHeader
    PAYLOAD = None

    def __init__(self, fixed: MQTTFixedHeader=None, variable_header: PacketIdVariableHeader=None, payload=None):
        if fixed is None:
            header = MQTTFixedHeader(UNSUBACK, 0x00)
        else:
            if fixed.packet_type is not UNSUBACK:
                raise DistMQTTException("Invalid fixed packet type %s for UnsubackPacket init" % fixed.packet_type)
            header = fixed

        super().__init__(header)
        self.variable_header = variable_header
        self.payload = payload

    @classmethod
    def build(cls, packet_id):
        variable_header = PacketIdVariableHeader(packet_id)
        return cls(variable_header=variable_header)
